#include "AngelCodeFont.h"
#include <fstream>
#include <sstream>

static const float WIDTH = 800.0f;
static const float HEIGHT = 600;

bool AngelCodeFont::parseFont(const char *filename, Charset *charset)
{
  std::ifstream stream(filename);

	std::string line;
	std::string read, key, value;
	std::size_t i;
	while(!stream.eof())
	{
		std::stringstream line_stream;
		std::getline(stream, line);
		line_stream << line;

		// read the line's type
		line_stream >> read;
		if(read == "common")
		{
			// this holds common data
			while(!line_stream.eof())
			{
				std::stringstream converter;
				line_stream >> read;
				i = read.find('=');
				key = read.substr(0, i);
				value = read.substr(i + 1);

				//assign the correct value
				converter << value;
				if(key == "lineheight")
					converter >> charset->LineHeight;
				else if(key == "base")
					converter >> charset->Base;
				else if(key == "scaleW")
					converter >> charset->Width;
				else if(key == "scaleH")
					converter >> charset->Height;
				else if(key == "pages")
					converter >> charset->Pages;
			}
		}
		else if(read == "char")
		{
			//this is data for a specific char
			unsigned short charid = 0;

			while(!line_stream.eof())
			{
				std::stringstream converter;
				line_stream >> read;
				i = read.find('=');
				key = read.substr(0, i);
				value = read.substr(i + 1);

				// assign the correct value
				converter << value;
				if(key == "id")
					converter >> charid;
				else if(key == "x")
					converter >> charset->Chars[charid].x;
				else if(key == "y")
					converter >> charset->Chars[charid].y;
				else if(key == "width")
					converter >> charset->Chars[charid].Width;
				else if(key == "height")
					converter >> charset->Chars[charid].Height;
				else if(key == "xoffset")
					converter >> charset->Chars[charid].XOffset;
				else if(key == "yoffset")
					converter >> charset->Chars[charid].YOffset;
				else if(key == "xadvance")
					converter >> charset->Chars[charid].XAdvance;
				else if(key == "page")
					converter >> charset->Chars[charid].Page;
			}
		}
	}

  stream.close();

	return true;
}

void AngelCodeFont::showText(const char *text, float x, float y, 
                             float colR, float colG, float colB,
                             ResHandle fontMaterialRes, int layer,
                             AngelCodeFont::Charset *charset)
{
  for(; *text; text++)
  {
    CharDescriptor *chardesc = &charset->Chars[*text];

    // upper left
    float ul_tu = chardesc->x / charset->Width;
    float ul_tv = chardesc->y / charset->Height;
    float ul_x = x + chardesc->XOffset / WIDTH;
    float ul_y = y + chardesc->YOffset / HEIGHT;

    // upper right
    float ur_tu = (chardesc->x + chardesc->Width) / charset->Width;
    float ur_tv = chardesc->y / charset->Height;
    float ur_x = x + (chardesc->Width + chardesc->XOffset) / WIDTH;
    float ur_y = y + chardesc->YOffset / HEIGHT;

    // bottom right
    float br_tu = (chardesc->x + chardesc->Width) / charset->Width;
    float br_tv = (chardesc->y + chardesc->Height) / charset->Height;
    float br_x = x + (chardesc->Width + chardesc->XOffset) / WIDTH;
    float br_y = y + (chardesc->Height + chardesc->YOffset) / HEIGHT;

    // bottom left
    float bl_tu = chardesc->x / charset->Width;
    float bl_tv = (chardesc->y + chardesc->Height) / charset->Height;
    float bl_x = x + chardesc->XOffset / WIDTH;
    float bl_y = y + (chardesc->Height + chardesc->YOffset) / HEIGHT;

    Horde3D::showOverlay(
      ul_x, ul_y, ul_tu, 1.0f-ul_tv, 
      bl_x, bl_y, bl_tu, 1.0f-bl_tv, 
      br_x, br_y, br_tu, 1.0f-br_tv, 
      ur_x, ur_y, ur_tu, 1.0f-ur_tv, 
      colR, colG, colB, 1,
      fontMaterialRes, layer);

    x += chardesc->XAdvance / WIDTH;
  }
}
